import pandas as pd
from decimal import Decimal, ROUND_HALF_UP
pd.set_option('display.max_rows', None)  # 显示完整的行
pd.set_option('display.expand_frame_repr', False)  # 设置不折叠数据
def cal_fuquan_price(df, fuquan_type='前复权'):
    """
    用于计算复权价格
    :param df: 必须包含的字段：收盘价，前收盘价，开盘价，最高价，最低价
    :param fuquan_type: ‘前复权’或者‘后复权’
    :return: 最终输出的df中，新增字段：收盘价_复权，开盘价_复权，最高价_复权，最低价_复权
    """

    # 计算复权因子：假设你一开始有1元钱，投资到这个股票，最终会变成多少钱.
    # 注:不能用 df['复权因子'] = (df['收盘价'] / df['前收盘价'] - 1).cumprod() 因为上市首日有的数据里前收盘价为0
    df['复权因子'] = (1 + df['涨跌幅'] / 100).cumprod()

    # 计算前复权、后复权收盘价
    if fuquan_type == '后复权':
        df['收盘价_复权'] = df['复权因子'] * (df.iloc[0]['收盘价'] / df.iloc[0]['复权因子'])
    elif fuquan_type == '前复权':
        df['收盘价_复权'] = df['复权因子'] * (df.iloc[-1]['收盘价'] / df.iloc[-1]['复权因子'])
    else:
        raise ValueError('计算复权价时，出现未知的复权类型：%s' % fuquan_type)

    # 计算复权
    df['开盘价_复权'] = df['开盘价'] / df['收盘价'] * df['收盘价_复权']
    df['最高价_复权'] = df['最高价'] / df['收盘价'] * df['收盘价_复权']
    df['最低价_复权'] = df['最低价'] / df['收盘价'] * df['收盘价_复权']
    del df['复权因子']
    return df

def cal_zdt_price(df):
    """
    计算股票当天的涨跌停价格。在计算涨跌停价格的时候，按照严格的四舍五入。
    包含st股，但是不包含新股
    涨跌停制度规则:
        ---2020年8月3日
        非ST股票 10%
        ST股票 5%

        ---2020年8月4日至今
        普通非ST股票 10%
        普通ST股票 5%

        科创板（sh68） 20%
        创业板（sz30） 20%
        科创板和创业板即使ST，涨跌幅限制也是20%

        北交所（bj） 30%

    :param df: 必须得是日线数据。必须包含的字段：前收盘价，开盘价，最高价，最低价
    :return:
    """
    # 计算涨停价格
    # 普通股票
    cond_st = (df['股票名称'].str.contains('ST') | df['股票名称'].str.contains('st'))
    df['涨停价'] = df['前收盘价'] * 1.1
    df['跌停价'] = df['前收盘价'] * 0.9
    df.loc[cond_st, '涨停价'] = df['前收盘价'] * 1.05
    df.loc[cond_st, '跌停价'] = df['前收盘价'] * 0.95

    # 2020年8月23日之后涨跌停规则有所改变
    # 新规的科创板
    new_rule_kcb = (df['交易日期'] > pd.to_datetime('2020-08-23')) & df['股票代码'].str.contains('sh68')
    # 新规的创业板
    new_rule_cyb = (df['交易日期'] > pd.to_datetime('2020-08-23')) & df['股票代码'].str.contains('sz30')
    # 北交所条件
    cond_bj = df['股票代码'].str.contains('bj')

    # 科创板 & 创业板
    df.loc[new_rule_kcb | new_rule_cyb, '涨停价'] = df['前收盘价'] * 1.2
    df.loc[new_rule_kcb | new_rule_cyb, '跌停价'] = df['前收盘价'] * 0.8

    # 北交所
    df.loc[cond_bj, '涨停价'] = df['前收盘价'] * 1.3
    df.loc[cond_bj, '跌停价'] = df['前收盘价'] * 0.7

    # 四舍五入
    df['涨停价'] = df['涨停价'].apply(
        lambda x: float(Decimal(x + 1e-7).quantize(Decimal('1.00'), rounding=ROUND_HALF_UP)))
    df['跌停价'] = df['跌停价'].apply(
        lambda x: float(Decimal(x + 1e-7).quantize(Decimal('1.00'), rounding=ROUND_HALF_UP)))

    # 判断是否一字涨停
    df['一字涨停'] = 0
    df.loc[df['最低价'] >= df['涨停价'], '一字涨停'] = 1
    # 判断是否一字跌停
    df['一字跌停'] = 0
    df.loc[df['最高价'] <= df['跌停价'], '一字跌停'] = 1
    # 判断是否开盘涨停
    df['开盘涨停'] = 0
    df.loc[df['开盘价'] >= df['涨停价'], '开盘涨停'] = 1
    # 判断是否开盘跌停
    df['开盘跌停'] = 0
    df.loc[df['开盘价'] <= df['跌停价'], '开盘跌停'] = 1
    # 判断是否涨停收盘，涨停不能买
    df['收盘涨停'] = 0
    df.loc[df['收盘价'] >= df['涨停价'], '收盘涨停'] = 1
    # 判断是否收盘跌停，跌停可买，跌停不卖
    df['收盘跌停'] = 0
    df.loc[df['收盘价'] <= df['跌停价'], '收盘跌停'] = 1

    # 判断涨停炸板
    df['涨停炸板'] = 0
    df.loc[(df['最高价'] == df['涨停价']) & (df['收盘价'] < df['涨停价']), '涨停炸板'] = 1

    return df

trade_date = '20231229'
path = 'sz002103.csv'
sb_pct = 0.995  # 最高价/涨停价，即最高价达到涨停价的99.5%，最高价差涨停价约0.5%时扫板

df = pd.read_csv(path, parse_dates=['交易日期'], dtype={'收盘价': float, '前收盘价': float})

df['上市天数'] = df.index + 1
# 只取20100101之后+上市30天后的日k线数据,因计算封板率，数据不宜太少
df = df[(df['交易日期'] >= pd.to_datetime('20100101')) & (df['上市天数'] > 30)]
# 去重复值
df.drop_duplicates(subset=['交易日期'], keep='last', inplace=True)
df.reset_index(drop=True, inplace=True)
df = cal_fuquan_price(df, fuquan_type='前复权')
df = cal_zdt_price(df)

# 统计2010年且上市一年后到trade_date的触板次数和封板率
df_c0 = df.copy()
df_trade = df_c0[df_c0['交易日期'] <= pd.to_datetime(trade_date)]
df_p = df_trade.copy()
df_h = df_trade.copy()

yzzt_sum = df_trade['一字涨停'].sum()
chuban_num = df_trade['收盘涨停'].sum() - df_trade['一字涨停'].sum() + df_trade['涨停炸板'].sum()

df_h = df_h[df_h['最高价'] / df_h['涨停价'] >= sb_pct]
high995_sum = len(df_h)
high_995_to_zt_num = high995_sum - yzzt_sum
# print(high995_sum)
df_p['次日涨跌幅'] = None
df_p['次日涨跌幅'] = df_p['涨跌幅'].shift(-1)
df_p = df_p[0:-1]
df_zt = df_p[(df_p['收盘涨停'] == 1) & (df_p['一字涨停'] == 0)]
pct_sum = df_zt['次日涨跌幅'].sum()

# 排除掉一字板（包含一字板会导致真实封板率偏高）
zt_sum = df_trade['收盘涨停'].sum() - df_trade['一字涨停'].sum()
df['封板次数（排除一字板）'] = None
df.loc[df['交易日期'] == pd.to_datetime(trade_date), '封板次数（排除一字板）'] = zt_sum

df['触板次数'] = None
df.loc[df['交易日期'] == pd.to_datetime(trade_date), '触板次数'] = chuban_num
df['封板率'] = None
if chuban_num > 0:
    df.loc[df['交易日期'] == pd.to_datetime(trade_date), '封板率'] = zt_sum / chuban_num
df['次日平均涨跌幅'] = None
if zt_sum > 0:
    df.loc[df['交易日期'] == pd.to_datetime(trade_date), '次日平均涨跌幅'] = pct_sum / zt_sum
df['扫板封板胜率'] = None
if high_995_to_zt_num > 0:
    df.loc[df['交易日期'] == pd.to_datetime(trade_date), '扫板封板胜率'] = zt_sum / high_995_to_zt_num
# print(df.columns)
df = df[['交易日期', '股票代码', '股票名称', '一字涨停', '一字跌停', '开盘涨停', '开盘跌停', '收盘涨停', '收盘跌停', '涨停炸板',
       '触板次数', '封板次数（排除一字板）', '封板率', '次日平均涨跌幅', '扫板封板胜率']]
print(df.tail(1))


